/*
  Create or edit Perforce user specifications and preferences.

  There are three types of Perforce users: standard users, operator users, and service users.
  Standard users are the default, and each standard user consumes one Perforce license.
  The operator user type is intended for system administrators; they are subject to the same restrictions on permissions as any other user, but are further restricted in that they can run only a limited subset of Perforce commands.
  Service users are intended for inter-server communication in replicated and multi-server environments, and are restricted to an even smaller subset of Perforce commands.
  Neither operators nor service users consume Perforce licenses.

*/
function UserCommand(data) {

    Object.defineProperties(this, {
        /*
          The Perforce username.
        */
        "user": {
            value: data ? data.user : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Type of user: standard, operator, or service.

          Once you set the type, you cannot change it.

        */
        "type": {
            value: data ? data.type : undefined,
            enumerable: true,
            writable: true
        },
        /*
          One of the following: perforce or ldap.

          Specifying perforce enables authentication using Perforce’s internal db.user table or by way of an authentication trigger.
          This is the default unless it is overridden with the auth.default.method configurable.

          Specifying ldap enables authentication against AD/LDAP servers specified by the currently active LDAP configurations.

        */
        "authMethod": {
            value: data ? data.authMethod : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The user’s email address. By default, this is user@client.

        */
        "email": {
            value: data ? data.email : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The date and time this specification was last updated.

        */
        "update": {
            enumerable: true,
            get: function() {
                if (data && data.update) {
                    var strVal = data.update;
                    return Date.parse(strVal);
                }
            }
        },
        /*
          The date and time this user last ran a Perforce command.

        */
        "access": {
            enumerable: true,
            get: function() {
                if (data && data.access) {
                    var strVal = data.access;
                    return Date.parse(strVal);
                }
            }
        },
        /*
          The user's full name.
        */
        "fullName": {
            value: data ? data.fullName : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Jobs matching this jobview appear on any changelists created by this user.
          Jobs that are fixed by the changelist should be left in the changelist when it's submitted with p4 submit; other jobs should be deleted from the form before submission.

        */
        "jobView": {
            value: data ? data.jobView : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The user's password.
        */
        "password": {
            value: data ? data.password : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The date and time of the user’s last password change.
          If the user has no password, this field is blank.

        */
        "passwordChange": {
            enumerable: true,
            get: function() {
                if (data && data.passwordChange) {
                    var strVal = data.passwordChange;
                    return Date.parse(strVal);
                }
            }
        },
        /*
          A list of files the user would like to review.
          This field can include exclusionary mappings.

        */
        "reviews": {
            value: data ? data.reviews : undefined,
            enumerable: true,
            writable: true
        }
    });

}

module.exports = UserCommand;
